﻿using System;
using System.Collections.Generic;
using Marten.Generation;

namespace Marten.Schema
{
    public interface IDbObjects
    {
        /// <summary>
        /// Fetches a list of all of the Marten generated tables
        /// in the database
        /// </summary>
        /// <returns></returns>
        TableName[] SchemaTables();

        /// <summary>
        /// Fetches a list of the Marten document tables
        /// in the database
        /// </summary>
        /// <returns></returns>
        TableName[] DocumentTables();

        /// <summary>
        /// Fetches a list of functions generated by Marten
        /// in the database
        /// </summary>
        /// <returns></returns>
        FunctionName[] SchemaFunctionNames();

        bool TableExists(TableName table);


        IEnumerable<ActualIndex> AllIndexes();


        IEnumerable<ActualIndex> IndexesFor(TableName table);

        FunctionBody DefinitionForFunction(FunctionName function);


        /// <summary>
        /// Fetch the actual table model for a document type
        /// </summary>
        /// <param name="documentMapping"></param>
        /// <returns></returns>
        TableDefinition TableSchema(IDocumentMapping documentMapping);

        TableDefinition TableSchema(Type documentType);

        SchemaObjects FindSchemaObjects(DocumentMapping mapping);
    }
}